@@ -166,6 +166,14 @@ remove_source (GcalManager  *self,
   g_return_if_fail (E_IS_SOURCE (source));
 
   calendar = g_hash_table_lookup (self->clients, source);
+
+  if (!calendar)
+    return;
+
+  GCAL_TRACE_MSG ("Removing calendar %s (%s)",
+                  e_source_get_display_name (source),
+                  e_source_get_uid (source));
+
   g_object_ref (calendar);
 
   e_cal_data_model_remove_client (self->e_data_model,
@@ -258,11 +266,13 @@ on_calendar_created_cb (GObject      *source_object,
   self = GCAL_MANAGER (user_data);
   calendar = gcal_calendar_new_finish (result, &error);
 
-  if (error &&
-      !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
-      !g_error_matches (error, GCAL_CALENDAR_ERROR, GCAL_CALENDAR_ERROR_NOT_CALENDAR))
+  if (error)
     {
-      g_warning ("Failed to open/connect to calendar: %s", error->message);
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+          !g_error_matches (error, GCAL_CALENDAR_ERROR, GCAL_CALENDAR_ERROR_NOT_CALENDAR))
+        {
+          g_warning ("Failed to open/connect to calendar: %s", error->message);
+        }
       return;
     }
 
