@@ -799,10 +799,10 @@ gcal_quick_add_popover_set_property (GObject      *object,
             on_calendar_added (manager, l->data, self);
 
           /* Connect to the manager signals and keep the list updates */
-          g_signal_connect (manager, "calendar-added", G_CALLBACK (on_calendar_added), self);
-          g_signal_connect (manager, "calendar-changed", G_CALLBACK (on_calendar_changed), self);
-          g_signal_connect (manager, "calendar-removed", G_CALLBACK (on_calendar_removed), self);
-          g_signal_connect_swapped (manager, "notify::default-calendar", G_CALLBACK (update_default_calendar_row), self);
+          g_signal_connect_object (manager, "calendar-added", G_CALLBACK (on_calendar_added), self, 0);
+          g_signal_connect_object (manager, "calendar-changed", G_CALLBACK (on_calendar_changed), self, 0);
+          g_signal_connect_object (manager, "calendar-removed", G_CALLBACK (on_calendar_removed), self, 0);
+          g_signal_connect_object (manager, "notify::default-calendar", G_CALLBACK (update_default_calendar_row), self, G_CONNECT_SWAPPED);
 
           g_signal_connect_object (self->context,
                                    "notify::time-format",
