@@ -662,7 +662,6 @@ edit_dialog_closed (GtkDialog *dialog,
   GcalEvent *event;
   GcalView *view;
   GList *widgets;
-  ESource *source;
 
   GCAL_ENTRY;
 
@@ -673,7 +672,6 @@ edit_dialog_closed (GtkDialog *dialog,
   view = GCAL_VIEW (window->views[window->active_view]);
   mod = GCAL_RECURRENCE_MOD_THIS_ONLY;
   calendar = gcal_event_get_calendar (event);
-  source = gcal_calendar_get_source (calendar);
 
   if (!gcal_edit_dialog_get_recurrence_changed (edit_dialog) &&
       gcal_event_has_recurrence (event) &&
@@ -681,7 +679,7 @@ edit_dialog_closed (GtkDialog *dialog,
        response != GTK_RESPONSE_CANCEL &&
        response != GTK_RESPONSE_DELETE_EVENT &&
        gcal_event_has_recurrence (event) &&
-       !ask_recurrence_modification_type (GTK_WIDGET (dialog), &mod, source)))
+       !ask_recurrence_modification_type (GTK_WIDGET (dialog), &mod, calendar)))
     {
       return;
     }
@@ -869,15 +867,15 @@ gcal_window_constructed (GObject *object)
    * FIXME: this is a hack around the issue that happens when trying to bind
    * these properties using the GtkBuilder .ui file.
    */
-  g_object_bind_property (self, "context", self->calendar_popover, "context", G_BINDING_DEFAULT);
+  g_object_bind_property (self, "context", self->calendar_popover, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
   g_object_bind_property (self, "context", self->weather_settings, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
-  g_object_bind_property (self, "context", self->calendar_management_dialog, "context", G_BINDING_DEFAULT);
+  g_object_bind_property (self, "context", self->calendar_management_dialog, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
   g_object_bind_property (self, "context", self->week_view, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
   g_object_bind_property (self, "context", self->month_view, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
   g_object_bind_property (self, "context", self->year_view, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
   g_object_bind_property (self, "context", self->edit_dialog, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
   g_object_bind_property (self, "context", self->quick_add_popover, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
-  g_object_bind_property (self, "context", self->search_button, "context", G_BINDING_DEFAULT);
+  g_object_bind_property (self, "context", self->search_button, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
   GCAL_EXIT;
 }
@@ -909,16 +907,15 @@ gcal_window_set_property (GObject      *object,
       break;
 
     case PROP_CONTEXT:
-      if (g_set_object (&self->context, g_value_get_object (value)))
-        {
-          g_settings_bind (gcal_context_get_settings (self->context),
-                           "active-view",
-                           self,
-                           "active-view",
-                           G_SETTINGS_BIND_SET | G_SETTINGS_BIND_GET);
+      g_assert (self->context == NULL);
+      self->context = g_value_dup_object (value);
+
+      g_settings_bind (gcal_context_get_settings (self->context),
+                       "active-view",
+                       self,
+                       "active-view",
+                       G_SETTINGS_BIND_SET | G_SETTINGS_BIND_GET);
 
-          g_object_notify_by_pspec (object, properties[PROP_CONTEXT]);
-        }
       break;
 
     default:
@@ -1025,7 +1022,7 @@ gcal_window_class_init (GcalWindowClass *klass)
                                                   "Context",
                                                   "Context",
                                                   GCAL_TYPE_CONTEXT,
-                                                  G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                                                  G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   properties[PROP_NEW_EVENT_MODE] = g_param_spec_boolean ("new-event-mode",
                                                           "New Event mode",
@@ -1133,6 +1130,7 @@ gcal_window_init (GcalWindow *self)
   gcal_window_add_accelerator (app, "win.change-view(2)",    "<Ctrl>2");
   gcal_window_add_accelerator (app, "win.change-view(3)",    "<Ctrl>3");
   gcal_window_add_accelerator (app, "app.quit", "<Ctrl>q");
+  gcal_window_add_accelerator (app, "win.new-event", "<Ctrl>n");
 }
 
 /**
