@@ -107,7 +107,12 @@ search_func (GTask        *task,
                               start,
                               end);
 
-  g_task_return_pointer (task, g_steal_pointer (&model), g_object_unref);
+  gcal_search_model_wait_for_hits (model, cancellable);
+
+  if (g_cancellable_is_cancelled (cancellable))
+    g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_CANCELLED, "Cancelled");
+  else
+    g_task_return_pointer (task, g_steal_pointer (&model), g_object_unref);
 }
 
 static void
@@ -177,14 +182,16 @@ gcal_search_engine_constructed (GObject *object)
 {
   GcalSearchEngine *self = (GcalSearchEngine *)object;
   GcalManager *manager;
+  GTimeZone *zone;
 
   G_OBJECT_CLASS (gcal_search_engine_parent_class)->constructed (object);
 
   /* Setup the data model */
   self->data_model = e_cal_data_model_new (gcal_thread_submit_job);
   e_cal_data_model_set_expand_recurrences (self->data_model, TRUE);
-  e_cal_data_model_set_timezone (self->data_model, e_cal_util_get_system_timezone ());
 
+  zone = gcal_context_get_timezone (self->context);
+  e_cal_data_model_set_timezone (self->data_model, gcal_timezone_to_icaltimezone (zone));
 
   manager = gcal_context_get_manager (self->context);
   g_signal_connect_object (manager, "calendar-added", G_CALLBACK (on_manager_calendar_added_cb), self, 0);
