@@ -118,10 +118,11 @@ update_grid_scroll_position (GcalWeekView *self)
 
       stack = gtk_widget_get_ancestor (GTK_WIDGET (self), GTK_TYPE_STACK);
 
-      g_signal_connect (stack,
-                        "notify::visible-child",
-                        G_CALLBACK (stack_visible_child_changed_cb),
-                        self);
+      g_signal_connect_object (stack,
+                               "notify::visible-child",
+                               G_CALLBACK (stack_visible_child_changed_cb),
+                               self,
+                               0);
 
       self->scroll_grid_timeout_id = 0;
 
@@ -569,6 +570,7 @@ gcal_week_view_set_property (GObject       *object,
       break;
 
     case PROP_CONTEXT:
+      g_assert (self->context == NULL);
       self->context = g_value_dup_object (value);
 
       gcal_week_grid_set_context (GCAL_WEEK_GRID (self->week_grid), self->context);
@@ -579,8 +581,6 @@ gcal_week_view_set_property (GObject       *object,
                                G_CALLBACK (gtk_widget_queue_draw),
                                self->hours_bar,
                                G_CONNECT_SWAPPED);
-
-      g_object_notify (object, "context");
       break;
 
     default:
